# Generated by Django 4.0.4 on 2022-04-29 18:11

from django.db import migrations, models
import django.utils.timezone
import signal


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SniffingJobModel',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('json_file', models.CharField(max_length=250)),
                ('status', models.PositiveIntegerField(default=signal.Signals['SIGCONT'])),
                ('pid', models.PositiveIntegerField()),
                ('_pid_file', models.CharField(max_length=250)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('filters', models.TextField(null=True)),
                ('_interfaces', models.TextField()),
                ('pcap_file', models.CharField(max_length=250, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WebParsingJobModel',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('json_file', models.CharField(max_length=250)),
                ('status', models.PositiveIntegerField(default=signal.Signals['SIGCONT'])),
                ('pid', models.PositiveIntegerField()),
                ('_pid_file', models.CharField(max_length=250)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('url', models.CharField(max_length=512)),
                ('_parsing_type', models.CharField(choices=[('single_page', 'Single Page'), ('website_crawling', 'Website Crawling')], max_length=50)),
                ('_parsing_tags', models.CharField(choices=[('all_parse', 'All Tags'), ('relevant_parse', 'Relevant Tags (a, script, link, Form Tags, Meta Tags)'), ('form_parse', 'Form Tags (form, input, textarea, select, option)'), ('meta_parse', 'Meta Tags (meta, xmp)')], max_length=50)),
                ('depth', models.IntegerField()),
                ('cookies', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
